@@ -1,4 +1,6 @@
-//! `process::Output` assertions.
+//! [`Output`][Output] assertions.
+//!
+//! [Output]: https://doc.rust-lang.org/std/process/struct.Output.html
 
 use std::fmt;
 use std::process;
@@ -6,11 +8,13 @@ use std::str;
 
 use predicates;
 use predicates::str::PredicateStrExt;
+use predicates_core;
+use predicates_tree::CaseTreeExt;
 
-use cmd::dump_buffer;
+use errors::dump_buffer;
 use errors::output_fmt;
 
-/// Assert the state of an `Output`.
+/// Assert the state of an [`Output`].
 ///
 /// # Examples
 ///
@@ -24,8 +28,10 @@ use errors::output_fmt;
 ///     .assert()
 ///     .success();
 /// ```
+///
+/// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
 pub trait OutputAssertExt {
-    /// Wrap with an interface for that provides assertions on the `process::Output`.
+    /// Wrap with an interface for that provides assertions on the [`Output`].
     ///
     /// # Examples
     ///
@@ -39,6 +45,8 @@ pub trait OutputAssertExt {
     ///     .assert()
     ///     .success();
     /// ```
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
     fn assert(self) -> Assert;
 }
 
@@ -51,13 +59,13 @@ impl OutputAssertExt for process::Output {
 impl<'c> OutputAssertExt for &'c mut process::Command {
     fn assert(self) -> Assert {
         let output = self.output().unwrap();
-        Assert::new(output).set_cmd(format!("{:?}", self))
+        Assert::new(output).append_context("command", format!("{:?}", self))
     }
 }
 
-/// Assert the state of an `Output`.
+/// Assert the state of an [`Output`].
 ///
-/// Create an `Assert` through the `OutputAssertExt` trait.
+/// Create an `Assert` through the [`OutputAssertExt`] trait.
 ///
 /// # Examples
 ///
@@ -71,49 +79,55 @@ impl<'c> OutputAssertExt for &'c mut process::Command {
 ///     .assert()
 ///     .success();
 /// ```
-#[derive(Debug)]
+///
+/// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+/// [`OutputAssertExt`]: trait.OutputAssertExt.html
 pub struct Assert {
     output: process::Output,
-    cmd: Option<String>,
-    stdin: Option<Vec<u8>>,
+    context: Vec<(&'static str, Box<fmt::Display>)>,
 }
 
 impl Assert {
-    /// Create an `Assert` for a given `Output`.
+    /// Create an `Assert` for a given [`Output`].
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
     pub fn new(output: process::Output) -> Self {
         Self {
             output,
-            cmd: None,
-            stdin: None,
+            context: vec![],
         }
     }
 
-    /// Add the command line for additional context.
-    pub fn set_cmd(mut self, cmd: String) -> Self {
-        self.cmd = Some(cmd);
+    /// Clarify failures with additional context.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use assert_cmd::prelude::*;
+    ///
+    /// use std::process::Command;
+    ///
+    /// Command::main_binary()
+    ///     .unwrap()
+    ///     .assert()
+    ///     .append_context("main", "no args")
+    ///     .success();
+    /// ```
+    pub fn append_context<D>(mut self, name: &'static str, context: D) -> Self
+    where
+        D: fmt::Display + 'static,
+    {
+        self.context.push((name, Box::new(context)));
         self
     }
 
-    /// Add the `stdn` for additional context.
-    pub fn set_stdin(mut self, stdin: Vec<u8>) -> Self {
-        self.stdin = Some(stdin);
-        self
-    }
-
-    /// Access the contained `std::process::Output`.
+    /// Access the contained [`Output`].
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
     pub fn get_output(&self) -> &process::Output {
         &self.output
     }
 
-    // How does user interact with assertion API?
-    // - On Assert class, using error chaining
-    //   - "Builder" or not?  If yes, then do we extend Result?
-    //   - How do we give a helpful unwrap?
-    // - Build up assertion data and "execute" it, like assert_cli used to?  But that was mostly
-    //   from building up before executing the command happened.  Now we're doing it
-    //   after-the-fact.
-    // - Immediately panic in each assertion? Let's give that a try.
-
     /// Ensure the command succeeded.
     ///
     /// # Examples
@@ -164,11 +178,7 @@ impl Assert {
     /// ```
     pub fn failure(self) -> Self {
         if self.output.status.success() {
-            panic!(
-                "Unexpected success\nstdout=```{}```\n{}",
-                dump_buffer(&self.output.stdout),
-                self
-            );
+            panic!("Unexpected success\n{}", self);
         }
         self
     }
@@ -176,11 +186,7 @@ impl Assert {
     /// Ensure the command aborted before returning a code.
     pub fn interrupted(self) -> Self {
         if self.output.status.code().is_some() {
-            panic!(
-                "Unexpected completion\nstdout=```{}```\n{}",
-                dump_buffer(&self.output.stdout),
-                self
-            );
+            panic!("Unexpected completion\n{}", self);
         }
         self
     }
@@ -189,40 +195,46 @@ impl Assert {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```rust,ignore
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
+    /// use predicates::prelude::*;
     ///
     /// Command::main_binary()
     ///     .unwrap()
     ///     .env("exit", "42")
     ///     .assert()
+    ///     .code(predicate::eq(42));
+    ///
+    /// // which can be shortened to:
+    /// Command::main_binary()
+    ///     .unwrap()
+    ///     .env("exit", "42")
+    ///     .assert()
     ///     .code(42);
     /// ```
+    ///
+    /// - See [`predicates::prelude`] for more predicates.
+    /// - See [`IntoCodePredicate`] for other built-in conversions.
+    ///
+    /// [`predicates::prelude`]: https://docs.rs/predicates/0.9.0/predicates/prelude
+    /// [`IntoCodePredicate`]: trait.IntoCodePredicate.html
     pub fn code<I, P>(self, pred: I) -> Self
     where
         I: IntoCodePredicate<P>,
-        P: predicates::Predicate<i32>,
+        P: predicates_core::Predicate<i32>,
     {
         self.code_impl(&pred.into_code())
     }
 
-    fn code_impl(self, pred: &predicates::Predicate<i32>) -> Self {
-        let actual_code = self.output.status.code().unwrap_or_else(|| {
-            panic!(
-                "Command interrupted\nstderr=```{}```\n{}",
-                dump_buffer(&self.output.stderr),
-                self
-            )
-        });
-        if !pred.eval(&actual_code) {
-            panic!(
-                "Unexpected return code\nstdout=```{}```\nstderr=```{}```\n{}",
-                dump_buffer(&self.output.stdout),
-                dump_buffer(&self.output.stderr),
-                self
-            );
+    fn code_impl(self, pred: &predicates_core::Predicate<i32>) -> Self {
+        let actual_code = self.output
+            .status
+            .code()
+            .unwrap_or_else(|| panic!("Command interrupted\n{}", self));
+        if let Some(case) = pred.find_case(false, &actual_code) {
+            panic!("Unexpected return code, failed {}\n{}", case.tree(), self);
         }
         self
     }
@@ -231,31 +243,46 @@ impl Assert {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```rust,ignore
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
+    /// use predicates::prelude::*;
     ///
     /// Command::main_binary()
     ///     .unwrap()
     ///     .env("stdout", "hello")
     ///     .env("stderr", "world")
     ///     .assert()
+    ///     .stdout(predicate::str::similar("hello\n").from_utf8());
+    ///
+    /// // which can be shortened to:
+    /// Command::main_binary()
+    ///     .unwrap()
+    ///     .env("stdout", "hello")
+    ///     .env("stderr", "world")
+    ///     .assert()
     ///     .stdout("hello\n");
     /// ```
+    ///
+    /// - See [`predicates::prelude`] for more predicates.
+    /// - See [`IntoOutputPredicate`] for other built-in conversions.
+    ///
+    /// [`predicates::prelude`]: https://docs.rs/predicates/0.9.0/predicates/prelude
+    /// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
     pub fn stdout<I, P>(self, pred: I) -> Self
     where
         I: IntoOutputPredicate<P>,
-        P: predicates::Predicate<[u8]>,
+        P: predicates_core::Predicate<[u8]>,
     {
         self.stdout_impl(&pred.into_output())
     }
 
-    fn stdout_impl(self, pred: &predicates::Predicate<[u8]>) -> Self {
+    fn stdout_impl(self, pred: &predicates_core::Predicate<[u8]>) -> Self {
         {
             let actual = &self.output.stdout;
-            if !pred.eval(actual) {
-                panic!("Unexpected stdout\n{}", self);
+            if let Some(case) = pred.find_case(false, &actual) {
+                panic!("Unexpected stdout, failed {}\n{}", case.tree(), self);
             }
         }
         self
@@ -265,31 +292,46 @@ impl Assert {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```rust,ignore
     /// use assert_cmd::prelude::*;
     ///
     /// use std::process::Command;
+    /// use predicates::prelude::*;
     ///
     /// Command::main_binary()
     ///     .unwrap()
     ///     .env("stdout", "hello")
     ///     .env("stderr", "world")
     ///     .assert()
+    ///     .stderr(predicate::str::similar("world\n").from_utf8());
+    ///
+    /// // which can be shortened to:
+    /// Command::main_binary()
+    ///     .unwrap()
+    ///     .env("stdout", "hello")
+    ///     .env("stderr", "world")
+    ///     .assert()
     ///     .stderr("world\n");
     /// ```
+    ///
+    /// - See [`predicates::prelude`] for more predicates.
+    /// - See [`IntoOutputPredicate`] for other built-in conversions.
+    ///
+    /// [`predicates::prelude`]: https://docs.rs/predicates/0.9.0/predicates/prelude
+    /// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
     pub fn stderr<I, P>(self, pred: I) -> Self
     where
         I: IntoOutputPredicate<P>,
-        P: predicates::Predicate<[u8]>,
+        P: predicates_core::Predicate<[u8]>,
     {
         self.stderr_impl(&pred.into_output())
     }
 
-    fn stderr_impl(self, pred: &predicates::Predicate<[u8]>) -> Self {
+    fn stderr_impl(self, pred: &predicates_core::Predicate<[u8]>) -> Self {
         {
             let actual = &self.output.stderr;
-            if !pred.eval(actual) {
-                panic!("Unexpected stderr\n{}", self);
+            if let Some(case) = pred.find_case(false, &actual) {
+                panic!("Unexpected stderr, failed {}\n\n{}", case.tree(), self);
             }
         }
         self
@@ -298,21 +340,23 @@ impl Assert {
 
 impl fmt::Display for Assert {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        if let Some(ref cmd) = self.cmd {
-            writeln!(f, "command=`{}`", cmd)?;
-        }
-        if let Some(ref stdin) = self.stdin {
-            if let Ok(stdin) = str::from_utf8(stdin) {
-                writeln!(f, "stdin=```{}```", stdin)?;
-            } else {
-                writeln!(f, "stdin=```{:?}```", stdin)?;
-            }
+        for &(ref name, ref context) in &self.context {
+            writeln!(f, "{}=`{}`", name, context)?;
         }
         output_fmt(&self.output, f)
     }
 }
 
-/// Used by `Assert::code` to convert `Self` into the needed `Predicate<i32>`.
+impl fmt::Debug for Assert {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("Assert")
+            .field("output", &self.output)
+            .finish()
+    }
+}
+
+/// Used by [`Assert::code`] to convert `Self` into the needed
+/// [`Predicate<i32>`].
 ///
 /// # Examples
 ///
@@ -320,22 +364,27 @@ impl fmt::Display for Assert {
 /// use assert_cmd::prelude::*;
 ///
 /// use std::process::Command;
+/// use predicates::prelude::*;
 ///
 /// Command::main_binary()
 ///     .unwrap()
 ///     .env("exit", "42")
 ///     .assert()
-///     .code(42);
-/// // which is equivalent to
+///     .code(predicate::eq(42));
+///
+/// // which can be shortened to:
 /// Command::main_binary()
 ///     .unwrap()
 ///     .env("exit", "42")
 ///     .assert()
-///     .code(predicates::ord::eq(42));
+///     .code(42);
 /// ```
+///
+/// [`Assert::code`]: struct.Assert.html#method.code
+/// [`Predicate<i32>`]: https://docs.rs/predicates-core/0.9.0/predicates_core/trait.Predicate.html
 pub trait IntoCodePredicate<P>
 where
-    P: predicates::Predicate<i32>,
+    P: predicates_core::Predicate<i32>,
 {
     /// The type of the predicate being returned.
     type Predicate;
@@ -346,7 +395,7 @@ where
 
 impl<P> IntoCodePredicate<P> for P
 where
-    P: predicates::Predicate<i32>,
+    P: predicates_core::Predicate<i32>,
 {
     type Predicate = P;
 
@@ -379,10 +428,39 @@ impl IntoCodePredicate<predicates::iter::InPredicate<i32>> for &'static [i32] {
     }
 }
 
-/// Used by `Assert` to convert Self into the needed `Predicate<[u8]>`.
+/// Used by [`Assert::stdout`] and [`Assert::stderr`] to convert Self
+/// into the needed [`Predicate<[u8]>`].
+///
+/// # Examples
+///
+/// ```rust,ignore
+/// use assert_cmd::prelude::*;
+///
+/// use std::process::Command;
+/// use predicates::prelude::*;
+///
+/// Command::main_binary()
+///     .unwrap()
+///     .env("stdout", "hello")
+///     .env("stderr", "world")
+///     .assert()
+///     .stdout(predicate::str::similar("hello\n").from_utf8());
+///
+/// // which can be shortened to:
+/// Command::main_binary()
+///     .unwrap()
+///     .env("stdout", "hello")
+///     .env("stderr", "world")
+///     .assert()
+///     .stdout("hello\n");
+/// ```
+///
+/// [`Assert::stdout`]: struct.Assert.html#method.stdout
+/// [`Assert::stderr`]: struct.Assert.html#method.stderr
+/// [`Predicate<[u8]>`]: https://docs.rs/predicates-core/0.9.0/predicates_core/trait.Predicate.html
 pub trait IntoOutputPredicate<P>
 where
-    P: predicates::Predicate<[u8]>,
+    P: predicates_core::Predicate<[u8]>,
 {
     /// The type of the predicate being returned.
     type Predicate;
@@ -393,7 +471,7 @@ where
 
 impl<P> IntoOutputPredicate<P> for P
 where
-    P: predicates::Predicate<[u8]>,
+    P: predicates_core::Predicate<[u8]>,
 {
     type Predicate = P;
 
@@ -403,7 +481,10 @@ where
 }
 
 // Keep `predicates` concrete Predicates out of our public API.
-/// Predicate used by `IntoOutputPredicate` for bytes
+/// [Predicate] used by [`IntoOutputPredicate`] for bytes.
+///
+/// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
+/// [Predicate]: https://docs.rs/predicates-core/0.9.0/predicates_core/trait.Predicate.html
 #[derive(Debug)]
 pub struct BytesContentOutputPredicate(predicates::ord::EqPredicate<&'static [u8]>);
 
@@ -414,10 +495,31 @@ impl BytesContentOutputPredicate {
     }
 }
 
-impl predicates::Predicate<[u8]> for BytesContentOutputPredicate {
+impl predicates_core::reflection::PredicateReflection for BytesContentOutputPredicate {
+    fn parameters<'a>(
+        &'a self,
+    ) -> Box<Iterator<Item = predicates_core::reflection::Parameter<'a>> + 'a> {
+        self.0.parameters()
+    }
+
+    /// Nested `Predicate`s of the current `Predicate`.
+    fn children<'a>(&'a self) -> Box<Iterator<Item = predicates_core::reflection::Child<'a>> + 'a> {
+        self.0.children()
+    }
+}
+
+impl predicates_core::Predicate<[u8]> for BytesContentOutputPredicate {
     fn eval(&self, item: &[u8]) -> bool {
         self.0.eval(item)
     }
+
+    fn find_case<'a>(
+        &'a self,
+        expected: bool,
+        variable: &[u8],
+    ) -> Option<predicates_core::reflection::Case<'a>> {
+        self.0.find_case(expected, variable)
+    }
 }
 
 impl fmt::Display for BytesContentOutputPredicate {
@@ -435,7 +537,11 @@ impl IntoOutputPredicate<BytesContentOutputPredicate> for &'static [u8] {
 }
 
 // Keep `predicates` concrete Predicates out of our public API.
-/// Predicate used by `IntoOutputPredicate` for `str`
+/// [Predicate] used by [`IntoOutputPredicate`] for [`str`].
+///
+/// [`IntoOutputPredicate`]: trait.IntoOutputPredicate.html
+/// [Predicate]: https://docs.rs/predicates-core/0.9.0/predicates_core/trait.Predicate.html
+/// [`str`]: https://doc.rust-lang.org/std/primitive.str.html
 #[derive(Debug, Clone)]
 pub struct StrContentOutputPredicate(
     predicates::str::Utf8Predicate<predicates::str::DifferencePredicate>,
@@ -448,10 +554,31 @@ impl StrContentOutputPredicate {
     }
 }
 
-impl predicates::Predicate<[u8]> for StrContentOutputPredicate {
+impl predicates_core::reflection::PredicateReflection for StrContentOutputPredicate {
+    fn parameters<'a>(
+        &'a self,
+    ) -> Box<Iterator<Item = predicates_core::reflection::Parameter<'a>> + 'a> {
+        self.0.parameters()
+    }
+
+    /// Nested `Predicate`s of the current `Predicate`.
+    fn children<'a>(&'a self) -> Box<Iterator<Item = predicates_core::reflection::Child<'a>> + 'a> {
+        self.0.children()
+    }
+}
+
+impl predicates_core::Predicate<[u8]> for StrContentOutputPredicate {
     fn eval(&self, item: &[u8]) -> bool {
         self.0.eval(item)
     }
+
+    fn find_case<'a>(
+        &'a self,
+        expected: bool,
+        variable: &[u8],
+    ) -> Option<predicates_core::reflection::Case<'a>> {
+        self.0.find_case(expected, variable)
+    }
 }
 
 impl fmt::Display for StrContentOutputPredicate {
@@ -479,7 +606,7 @@ mod test {
     fn convert_code<I, P>(pred: I) -> P
     where
         I: IntoCodePredicate<P>,
-        P: predicates::Predicate<i32>,
+        P: predicates_core::Predicate<i32>,
     {
         pred.into_code()
     }
@@ -513,7 +640,7 @@ mod test {
     fn convert_output<I, P>(pred: I) -> P
     where
         I: IntoOutputPredicate<P>,
-        P: predicates::Predicate<[u8]>,
+        P: predicates_core::Predicate<[u8]>,
     {
         pred.into_output()
     }
