@@ -1,7 +1,15 @@
 //! Simplify running `bin`s in a Cargo project.
 //!
+//! [`CommandCargoExt`] is an extension trait for [`Command`] to easily launch a crate's
+//! binaries.
+//!
+//! In addition, the underlying functions for looking up the crate's binaries are exposed to allow
+//! for optimizations, if needed.
+//!
 //! # Examples
 //!
+//! Simple case:
+//!
 //! ```rust
 //! use assert_cmd::prelude::*;
 //!
@@ -11,6 +19,24 @@
 //!     .unwrap()
 //!     .unwrap();
 //! ```
+//!
+//! Caching the binary's location:
+//!
+//! ```rust
+//! use assert_cmd::prelude::*;
+//!
+//! use std::process::Command;
+//!
+//! let bin_under_test = assert_cmd::cargo::main_binary_path().unwrap();
+//! Command::new(&bin_under_test)
+//!     .unwrap();
+//! ```
+//!
+//! Tip: Use [`lazy_static`] to cache `bin_under_test` across test functions.
+//!
+//! [`lazy_static`]: https://crates.io/crates/lazy_static
+//! [`CommandCargoExt`]: trait.CommandCargoExt.html
+//! [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
 
 use std::error::Error;
 use std::ffi;
@@ -20,12 +46,33 @@ use std::process;
 
 use escargot;
 
-/// Create a `Command` for a `bin` in the Cargo project.
+/// Create a [`Command`] for a `bin` in the Cargo project.
+///
+/// `CommandCargoExt` is an extension trait for [`Command`][Command] to easily launch a crate's
+/// binaries.
+///
+/// If the cargo overhead is too high per-call, you can cache the bin's location.  See the
+/// [`cargo`] module.
+///
+/// # Examples
+///
+/// ```rust
+/// use assert_cmd::prelude::*;
+///
+/// use std::process::Command;
+///
+/// Command::main_binary()
+///     .unwrap()
+///     .unwrap();
+/// ```
+///
+/// [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
+/// [`cargo`]: index.html
 pub trait CommandCargoExt
 where
     Self: Sized,
 {
-    /// Create a `Command` to run the crate's main binary.
+    /// Create a [`Command`] to run the crate's main binary.
     ///
     /// Note: only works if there one bin in the crate.
     ///
@@ -37,12 +84,14 @@ where
     /// use std::process::Command;
     ///
     /// Command::main_binary()
-    ///     .unwrap()
-    ///     .unwrap();
+    ///     .unwrap()  // get cargo binary
+    ///     .unwrap(); // run it
     /// ```
+    ///
+    /// [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
     fn main_binary() -> Result<Self, CargoError>;
 
-    /// Create a `Command` to run a specific binary of the current crate.
+    /// Create a [`Command`] to run a specific binary of the current crate.
     ///
     /// # Examples
     ///
@@ -55,9 +104,11 @@ where
     ///     .unwrap()
     ///     .unwrap();
     /// ```
+    ///
+    /// [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
     fn cargo_bin<S: AsRef<ffi::OsStr>>(name: S) -> Result<Self, CargoError>;
 
-    /// Create a `Command` to run a specific example of the current crate.
+    /// Create a [`Command`] to run a specific example of the current crate.
     ///
     /// # Examples
     ///
@@ -70,6 +121,8 @@ where
     ///     .unwrap()
     ///     .unwrap();
     /// ```
+    ///
+    /// [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
     fn cargo_example<S: AsRef<ffi::OsStr>>(name: S) -> Result<Self, CargoError>;
 }
 
@@ -126,12 +179,26 @@ fn extract_filenames(msg: &escargot::Message, kind: &str) -> Option<path::PathBu
 
 /// Get the path to the crate's main binary.
 ///
+/// Intended for caching the location, reducing the cargo overhead.
+///
 /// Note: only works if there one bin in the crate.
+///
+/// # Examples
+///
+/// ```rust
+/// use assert_cmd::prelude::*;
+///
+/// use std::process::Command;
+///
+/// let bin_under_test = assert_cmd::cargo::main_binary_path().unwrap();
+/// Command::new(&bin_under_test)
+///     .unwrap();
+/// ```
 pub fn main_binary_path() -> Result<path::PathBuf, CargoError> {
     let cargo = escargot::Cargo::new().build().current_release();
     let bins: Vec<_> = cargo
         .exec()
-        .map_err(|e| CargoError::with_cause(e))?
+        .map_err(CargoError::with_cause)?
         .filter_map(|m| extract_filenames(&m, "bin"))
         .collect();
     if bins.is_empty() {
@@ -146,11 +213,25 @@ pub fn main_binary_path() -> Result<path::PathBuf, CargoError> {
 }
 
 /// Get the path to the specified binary of the current crate.
+///
+/// Intended for caching the location, reducing the cargo overhead.
+///
+/// # Examples
+///
+/// ```rust
+/// use assert_cmd::prelude::*;
+///
+/// use std::process::Command;
+///
+/// let bin_under_test = assert_cmd::cargo::cargo_bin_path("bin_fixture").unwrap();
+/// Command::new(&bin_under_test)
+///     .unwrap();
+/// ```
 pub fn cargo_bin_path<S: AsRef<ffi::OsStr>>(name: S) -> Result<path::PathBuf, CargoError> {
     let cargo = escargot::Cargo::new().build().bin(name).current_release();
     let bins: Vec<_> = cargo
         .exec()
-        .map_err(|e| CargoError::with_cause(e))?
+        .map_err(CargoError::with_cause)?
         .filter_map(|m| extract_filenames(&m, "bin"))
         .collect();
     assert_eq!(bins.len(), 1);
@@ -158,6 +239,20 @@ pub fn cargo_bin_path<S: AsRef<ffi::OsStr>>(name: S) -> Result<path::PathBuf, Ca
 }
 
 /// Get the path to the specified example of the current crate.
+///
+/// Intended for caching the location, reducing the cargo overhead.
+///
+/// # Examples
+///
+/// ```rust
+/// use assert_cmd::prelude::*;
+///
+/// use std::process::Command;
+///
+/// let bin_under_test = assert_cmd::cargo::cargo_example_path("example_fixture").unwrap();
+/// Command::new(&bin_under_test)
+///     .unwrap();
+/// ```
 pub fn cargo_example_path<S: AsRef<ffi::OsStr>>(name: S) -> Result<path::PathBuf, CargoError> {
     let cargo = escargot::Cargo::new()
         .build()
@@ -165,7 +260,7 @@ pub fn cargo_example_path<S: AsRef<ffi::OsStr>>(name: S) -> Result<path::PathBuf
         .current_release();
     let bins: Vec<_> = cargo
         .exec()
-        .map_err(|e| CargoError::with_cause(e))?
+        .map_err(CargoError::with_cause)?
         .filter_map(|m| extract_filenames(&m, "example"))
         .collect();
     assert_eq!(bins.len(), 1);
