@@ -3,7 +3,9 @@ use std::fmt;
 use std::process;
 use std::str;
 
-/// `std::process::Output` represented as a `Result`.
+/// [`Output`] represented as a [`Result`].
+///
+/// Generally produced by [`OutputOkExt`].
 ///
 /// # Examples
 ///
@@ -17,9 +19,15 @@ use std::str;
 ///     .ok();
 /// assert!(result.is_ok());
 /// ```
+///
+/// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+/// [`Result`]: https://doc.rust-lang.org/std/result/enum.Result.html
+/// [`OutputOkExt`]: trait.OutputOkExt.html
 pub type OutputResult = Result<process::Output, OutputError>;
 
-/// `Command` error.
+/// [`Command`] error.
+///
+/// Generally produced by [`OutputOkExt`].
 ///
 /// # Examples
 ///
@@ -33,6 +41,9 @@ pub type OutputResult = Result<process::Output, OutputError>;
 ///     .env("exit", "42")
 ///     .unwrap_err();
 /// ```
+///
+/// [`Command`]: https://doc.rust-lang.org/std/process/struct.Command.html
+/// [`OutputOkExt`]: trait.OutputOkExt.html
 #[derive(Debug)]
 pub struct OutputError {
     cmd: Option<String>,
@@ -41,7 +52,10 @@ pub struct OutputError {
 }
 
 impl OutputError {
-    /// Convert `std::process::Output` into an `Error`.
+    /// Convert [`Output`] into an [`Error`].
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
+    /// [`Error`]: https://doc.rust-lang.org/std/error/trait.Error.html
     pub fn new(output: process::Output) -> Self {
         Self {
             cmd: None,
@@ -50,7 +64,9 @@ impl OutputError {
         }
     }
 
-    /// For errors that happen in creating a `std::process::Output`.
+    /// For errors that happen in creating a [`Output`].
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
     pub fn with_cause<E>(cause: E) -> Self
     where
         E: Error + Send + Sync + 'static,
@@ -74,7 +90,7 @@ impl OutputError {
         self
     }
 
-    /// Access the contained `std::process::Output`.
+    /// Access the contained [`Output`].
     ///
     /// # Examples
     ///
@@ -92,6 +108,8 @@ impl OutputError {
     ///     .unwrap();
     /// assert_eq!(Some(42), output.status.code());
     /// ```
+    ///
+    /// [`Output`]: https://doc.rust-lang.org/std/process/struct.Output.html
     pub fn as_output(&self) -> Option<&process::Output> {
         match self.cause {
             OutputCause::Expected(ref e) => Some(&e.output),
@@ -106,13 +124,14 @@ impl Error for OutputError {
     }
 
     fn cause(&self) -> Option<&Error> {
-        if let &OutputCause::Unexpected(ref err) = &self.cause {
+        if let OutputCause::Unexpected(ref err) = self.cause {
             Some(err.as_ref())
         } else {
             None
         }
     }
 }
+
 impl fmt::Display for OutputError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         if let Some(ref cmd) = self.cmd {
@@ -144,7 +163,6 @@ impl fmt::Display for OutputCause {
     }
 }
 
-/// Wrap `Output` to be `Dislay`able.
 #[derive(Debug)]
 struct Output {
     output: process::Output,
@@ -162,16 +180,47 @@ pub(crate) fn output_fmt(output: &process::Output, f: &mut fmt::Formatter) -> fm
     } else {
         writeln!(f, "code=<interrupted>")?;
     }
-    if let Ok(stdout) = str::from_utf8(&output.stdout) {
-        writeln!(f, "stdout=```{}```", stdout)?;
-    } else {
-        writeln!(f, "stdout=```{:?}```", output.stdout)?;
-    }
-    if let Ok(stderr) = str::from_utf8(&output.stderr) {
-        writeln!(f, "stderr=```{}```", stderr)?;
-    } else {
-        writeln!(f, "stderr=```{:?}```", output.stderr)?;
-    }
+
+    write!(f, "stdout=```")?;
+    write_buffer(&output.stdout, f)?;
+    writeln!(f, "```")?;
+
+    write!(f, "stderr=```")?;
+    write_buffer(&output.stderr, f)?;
+    writeln!(f, "```")?;
 
     Ok(())
 }
+
+pub(crate) fn dump_buffer(buffer: &[u8]) -> String {
+    if let Ok(buffer) = str::from_utf8(buffer) {
+        buffer.to_string()
+    } else {
+        format!("{:?}", buffer)
+    }
+}
+
+pub(crate) fn write_buffer(buffer: &[u8], f: &mut fmt::Formatter) -> fmt::Result {
+    if let Ok(buffer) = str::from_utf8(buffer) {
+        write!(f, "{}", buffer)
+    } else {
+        write!(f, "{:?}", buffer)
+    }
+}
+
+#[derive(Debug)]
+pub(crate) struct DebugBuffer {
+    buffer: Vec<u8>,
+}
+
+impl DebugBuffer {
+    pub(crate) fn new(buffer: Vec<u8>) -> Self {
+        DebugBuffer { buffer }
+    }
+}
+
+impl fmt::Display for DebugBuffer {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write_buffer(&self.buffer, f)
+    }
+}
